@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2015, 2016 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,9 @@
 #ifndef WHISKERMENU_ELEMENT_H
 #define WHISKERMENU_ELEMENT_H
 
+#include <climits>
+
+#include <exo/exo.h>
 #include <gdk/gdk.h>
 
 namespace WhiskerMenu
@@ -28,25 +31,23 @@ class Query;
 class Element
 {
 public:
-	Element() :
-		m_icon(NULL),
-		m_text(NULL),
-		m_tooltip(NULL),
-		m_sort_key(NULL)
-	{
-	}
-
 	virtual ~Element()
 	{
-		g_free(m_icon);
+		if (m_icon)
+		{
+			g_object_unref(m_icon);
+		}
 		g_free(m_text);
 		g_free(m_tooltip);
 		g_free(m_sort_key);
 	}
 
-	virtual int get_type() const = 0;
+	Element(const Element&) = delete;
+	Element(Element&&) = delete;
+	Element& operator=(const Element&) = delete;
+	Element& operator=(Element&&) = delete;
 
-	const gchar* get_icon() const
+	GIcon* get_icon() const
 	{
 		return m_icon;
 	}
@@ -65,9 +66,9 @@ public:
 	{
 	}
 
-	virtual guint search(const Query&)
+	virtual unsigned int search(const Query&)
 	{
-		return G_MAXUINT;
+		return UINT_MAX;
 	}
 
 	static bool less_than(const Element* lhs, const Element* rhs)
@@ -76,17 +77,9 @@ public:
 	}
 
 protected:
-	void set_icon(const gchar* icon)
-	{
-		g_free(m_icon);
-		m_icon = g_strdup(icon);
-	}
+	Element() = default;
 
-	void set_icon(gchar* icon)
-	{
-		g_free(m_icon);
-		m_icon = icon;
-	}
+	void set_icon(const gchar* icon);
 
 	void set_text(const gchar* text)
 	{
@@ -107,18 +100,14 @@ protected:
 	void set_tooltip(const gchar* tooltip)
 	{
 		g_free(m_tooltip);
-		m_tooltip = g_markup_escape_text(tooltip, -1);
+		m_tooltip = !exo_str_is_empty(tooltip) ? g_markup_escape_text(tooltip, -1) : nullptr;
 	}
 
 private:
-	Element(const Element&);
-	Element& operator=(const Element&);
-
-private:
-	gchar* m_icon;
-	gchar* m_text;
-	gchar* m_tooltip;
-	gchar* m_sort_key;
+	GIcon* m_icon = nullptr;
+	gchar* m_text = nullptr;
+	gchar* m_tooltip = nullptr;
+	gchar* m_sort_key = nullptr;
 };
 
 }
